from tcsoa.gen.Configuration._2010_04.services import SmartUiBldrService as imp0
from tcsoa.base import TcService


class SmartUiBldrService(TcService):
    getPropertiesForSelections = imp0.getPropertiesForSelections
    getResultForSelections = imp0.getResultForSelections
    getValuesForComponent = imp0.getValuesForComponent
    listForerunnersForComponents = imp0.listForerunnersForComponents
    listObjectsForComponents = imp0.listObjectsForComponents
    listValuesForComponents = imp0.listValuesForComponents
    ungetValuesForComponents = imp0.ungetValuesForComponents
