from tcsoa.gen.Internal_AuditManager._2012_09.services import ProcessHistoryManagementService as imp0
from tcsoa.base import TcService


class ProcessHistoryManagementService(TcService):
    getProcssHistoryAuditRecords = imp0.getProcssHistoryAuditRecords
