from tcsoa.gen.Internal_ProjectManagementAw._2019_12.services import ScheduleManagementAwService as imp0
from tcsoa.base import TcService


class ScheduleManagementAwService(TcService):
    manageProgramView = imp0.manageProgramView
